@@ -27,7 +27,7 @@ class TARGETError(Exception):
 
 import os
 import re
-import psutil
+#import psutil
 import shelve
 import subprocess
 import shutil
@@ -36,6 +36,62 @@ import datetime
 import logging
 from contextlib import closing
 
+
+class Target:
+    """
+    A target is a source or a destination.
+    Both of them can be a local directory
+    or a distant one though SSH
+    """
+    
+    def __init__(self, target):
+        self.logger = logging.getLogger('Vitalus.Target') 
+        self.logger.debug("Read target %s", target)
+        self.target = target
+        self.ttype = self._detect_target_type()
+
+        if self.is_dir():
+            self.path = target
+        elif self.is_ssh():
+            self.login, self.path = target.split(':')
+
+
+    def is_dir(self):
+        if self.ttype == 'DIR':
+            return True
+        else:
+            return False
+
+    def is_ssh(self):
+        if self.ttype == 'SSH':
+            return True
+        else:
+            return False
+
+    def _detect_target_type(self):
+        """
+        Return the target type
+        SSH if matches name@domaine.tld:dir
+        DIR if it's a directory
+       
+        :param target: Target (source or destination)
+        :raises TARGETError: weird target type 
+        """
+        if re.match('[a-zA-Z0-9+_\-\.]+@[0-9a-zA-Z][.-0-9a-zA-Z]*.[a-zA-Z]+\:.*', self.target):
+            #ssh
+            self.logger.debug("The target %s looks like SSH", self.target)
+            return 'SSH'
+        else:
+            if not os.path.exists(self.target):
+                self.logger.warn("The target %s does not exist", self.target)
+                self.logger.info('Aborting...')
+                raise TARGETError("Target %s does not exist" % self.target)
+            else:
+                self.logger.debug("The target %s looks like DIR", self.target)
+                return 'DIR'
+
+
+
 class Job:
     """
     Class containing a job
@@ -50,14 +106,20 @@ class Job:
     :param duration: How many days snapshots are kept
     :param keep: How many snapshots are (at least) kept
     :param filter: Rsync filters
+
+
+    Note:
+    -----
+    Source and destination path can be either real path
+    or a ssh login joined to the path by a : caracter.
     """
     #TODO signal...
 
     def __init__(self, log_dir, name, source, destination, period, snapshot, duration, keep, filter):
        
         self.name = name
-        self.source = source
-        self.destination = destination
+        self.source = Target(source)
+        self.destination = Target(destination)
         self.period = period
         self.snapshot = snapshot
         self.duration = duration
@@ -77,50 +139,37 @@ class Job:
         log_rotator = logging.handlers.TimedRotatingFileHandler(job_log, when='midnight', interval=1, backupCount=30, encoding=None, delay=False, utc=False)
         self.job_logger.addHandler(log_rotator)
         self.job_logger.setLevel(logging.INFO)
-        self.job_logger.info('='*20 + str(self.now) + '='*20)
         
-        #Source types
-        self.source_type = self._get_target_type(self.source)
-        self.dest_type = self._get_target_type(self.destination)
+        #Set previous and current backup paths
+        self.previous_backup_path = None
+        self.current_backup_path = None
 
-        self._check_disk_usage()
-
-    def _get_target_type(self, target):
-        """
-        Return the target type
-        SSH if matches name@domaine.tld:dir
-        DIR if it's a directory
-       
-        :param target: Target (source or destination)
-        :raises TARGETError: weird target type 
-        """
-        if re.match('[a-zA-Z0-9+_\-\.]+@[0-9a-zA-Z][.-0-9a-zA-Z]*.[a-zA-Z]+\:.*', target):
-            #ssh
-            self.logger.debug('The target looks like SSH')
-            return 'SSH'
+        last_date = self._get_last_backup()
+        self.current_backup_path = os.path.join(self.destination.path , self.name, str(self.current_date))
+        if last_date is None:
+            #It means that this is the first backup.
+            self.previous_backup_path = None
         else:
-            if not os.path.exists(target):
-                self.logger.warn("The target %s does not exist", target)
-                self.logger.info('Aborting...')
-                raise TARGETError("Target %s does not exist" % target)
-            else:
-                self.logger.debug('The target looks like DIR')
-                return 'DIR'
+            self.previous_backup_path = os.path.join(self.destination.path, self.name, str(last_date))
 
-    def _check_disk_usage(self):
-        """
-        Check the disk usage
-        :raises TARGETError: if low disk space
-        """
-        if self.dest_type == 'DIR':
-            #TODO, change the criterion
-            pass
-            #if psutil.disk_usage(self.destination)[2] < utils.get_folder_size(self.source): 
-            #    self.logger.critical("Low disk space: %s", self.destination)
-            #    raise TARGETError('Low disk space on %s' % self.destination)
-        elif self.dest_type == 'SSH':
-            #TODO
-            pass
+        self.logger.debug("Previous backup path: %s", self.previous_backup_path)
+        self.logger.debug("Current backup path: %s", self.current_backup_path)
+
+
+#    def _check_disk_usage(self):
+#        """
+#        Check the disk usage
+#        :raises TARGETError: if low disk space
+#        """
+#        if self.destination.is_dir():
+#            #TODO, change the criterion
+#            pass
+#            #if psutil.disk_usage(self.destination)[2] < utils.get_folder_size(self.source): 
+#            #    self.logger.critical("Low disk space: %s", self.destination)
+#            #    raise TARGETError('Low disk space on %s' % self.destination)
+#        elif self.destination.is_ssh():
+#            #TODO
+#            pass
 
     def _set_lastbackup_time(self):
         """
@@ -162,99 +211,116 @@ class Job:
             self.logger.debug("%s does not need backup", self.name)
             return False
 
-    def _delete_old_files(self, path, days=10, keep=10): 
+    def _delete_old_files(self, days=10, keep=10): 
         """
-        Delete old archives in a path
+        Delete old archives in the destination
 
-        :param path: path to clean
         :param days: delete files older than this value
         :param keep: keep at least this amount of archives
         """
         #TODO : review logs
 
+        path = os.path.join(self.destination.path, self.name)
         ####if self.terminate: return
-        filenames = [os.path.join(path, el) for el in os.listdir(path) ]
+        
+        if self.destination.is_dir():
+            filenames = [os.path.join(path, el) for el in os.listdir(path) ]
+        elif self.destination.is_ssh():
+            command = ['ssh', '-t', self.destination.login, 'ls', '-1', path]
+            self.logger.debug('SSH ls command: ' + str(command))
+            process = subprocess.Popen(command, bufsize=4096, stdout=subprocess.PIPE)
+            stdout, stderr = process.communicate()
+            filenames = stdout.decode()
+            filenames = filenames.split('\r\n')
+            filenames = [x for x in filenames if x!='']
+        else:
+            return
+
+        if self.destination.is_ssh():
+            #Not yet implemented!
+            return
+
         #delete first the older one
+        to_delete = []
         filenames.sort() #newer first
         nb_archives = len(filenames)
         for filepath in enumerate(filenames):
             #keep a sufficient number of archives
             if (nb_archives - filepath[0]) <= keep:
                 self.logger.debug('nb archives %s, keep %s, archive %s' % (nb_archives, keep, filepath[0]))
-                return
+                continue
             #Are they too old?
             file_date = datetime.datetime.fromtimestamp(os.path.getmtime(filepath[1]))
             if datetime.datetime.now() - file_date > datetime.timedelta(days=days): 
-                self.logger.info("remove %s", filepath[1])
-                try:
-                    shutil.rmtree(filepath[1])
-                except OSError:
-                    self.logger.error("Impossible to delete %s (symlink?)", filepath[1])
+
+                to_delete.append(filepath[1])
+                self.logger.info("Remove backup %s", filepath[1])
             else:
                 self.logger.debug("Keep backup %s", filepath[1])
-                return
+                continue
 
-    def _get_last_backup(self, path):
+        for element in to_delete:
+            if self.destination.is_dir():
+                print(element) #FIXME
+                try:
+                    shutil.rmtree(element)
+                except OSError:
+                    self.logger.error("Impossible to delete %s (symlink?)", element)
+            elif self.destination.is_ssh():
+                print(element) #FIXME
+
+    def _get_last_backup(self):
         """
         Get the last backup in path
         Return None if not available
-
-        :param path: path to look at
         """
-        #TODO : SSH look via ssh
-        if not os.path.isdir(path):
-            return None
-        filenames = [os.path.join(path, el) for el in os.listdir(path)]
+        path = os.path.join(self.destination.path, self.name)
+        if self.destination.is_dir():
+            if not os.path.isdir(path):
+                return None
+            filenames = [os.path.join(path, el) for el in os.listdir(path)]
+        elif self.destination.is_ssh():
+            #First, create at least the target if does not exists
+            command = ['ssh', '-t', self.destination.login, 'mkdir', '-p', path]
+            self.logger.debug('SSH mkdir command: ' + str(command))
+            process = subprocess.Popen(command, bufsize=4096, stdout=subprocess.PIPE)
+            stdout, stderr = process.communicate()
+            self.logger.debug('SSH mkdir result: ' + stdout.decode())
+
+
+            command = ['ssh', '-t', self.destination.login, 'ls', '-1', path]
+            self.logger.debug('SSH ls command: ' + str(command))
+            process = subprocess.Popen(command, bufsize=4096, stdout=subprocess.PIPE)
+            stdout, stderr = process.communicate()
+            filenames = stdout.decode()
+            filenames = filenames.split('\r\n')
+            filenames = [x for x in filenames if x!='']
+
         filenames.sort()
         if filenames == []:
             return None
-        return filenames[-1] 
+        try:
+            return filenames[-1] 
+        except IndexError:
+            return None
 
     def _prepare_destination(self): 
         """
         Prepare the destination to receive a backup:
-        * create dirs
-        * Initialize paths
+        Create dirs
         """
-        self.previous_backup_path = None
-        self.current_backup_path = None
-
-        if self.dest_type != 'SSH':
-            last_date = self._get_last_backup(os.path.join(self.destination, self.name))
-            self.current_backup_path = os.path.join(self.destination , self.name, str(self.current_date))
-            if last_date is None:
-                #It means that this is the first backup.
-                self.previous_backup_path = None
-            else:
-                self.previous_backup_path = os.path.join(self.destination, self.name, str(last_date))
-
+        if self.destination.is_dir():
             if self.snapshot:
                 os.makedirs(self.current_backup_path) #This one does not exist!
             else:
                 if self.previous_backup_path is None:
-                    #Create dir
-                    try:
-                        os.makedirs(self.current_backup_path) #This one may exist!
-                    except OSError:
-                        #they could already exist
-                        pass
+                    os.makedirs(self.current_backup_path, exist_ok=True) 
                 else:
-                    #Move dir
+                    #Move dir to set the new date in the path
                     os.rename(self.previous_backup_path, self.current_backup_path)
-        else:
-            # Create dirs on the server
-            login, dest_dir_path = destination.split(':')
-            #add ~/ if does not start with /
-            if not dest_dir_path.startswith('/'):
-                dest_dir_path = os.path.join('~', dest_dir_path)
-            if self.snapshot:
-                #For the moment, we do not support snapshots via SSH
-                pass
-            else:
-                self.current_backup_path = os.path.join(dest_dir_path, self.name)
-
+        elif self.destination.is_ssh():
             #Create dirs
-            command = ['ssh', '-t', login, 'mkdir', '-p', self.current_backup_path]
+            command = ['ssh', '-t', self.destination.login, 'mkdir', '-p', self.current_backup_path]
             self.logger.debug('SSH mkdir command: ' + str(command))
             process = subprocess.Popen(command, bufsize=4096, stdout=subprocess.PIPE)
             stdout, stderr = process.communicate()
@@ -283,16 +349,23 @@ class Job:
         command.append('--delete-excluded')
 
         # z: compress the flux if transfert thought a network
-        if (self.source_type or self.dest_type) == 'SSH':
+        if (self.source.is_ssh() or self.destination.is_ssh()):
             command.append('-z')
-        else: #FIXME for the moment, no snapshots though SSH !
-            # link-dest: write snapshots with hard-links
-            if self.snapshot and self.previous_backup_path is not None:
-                command.append('--link-dest=' + self.previous_backup_path)
+        if self.snapshot and self.previous_backup_path is not None:
+            #Even if it works for ttype==Dir
+            #It fails for ttype=SSH
+            #If link-dest is not a relative path
+            path = os.path.basename(self.previous_backup_path)
+            command.append('--link-dest=../' + path)
+
 
         # Add source and destination
-        command.append(self.source)
-        command.append(self.current_backup_path)
+        command.append(self.source.target)
+        if self.destination.is_ssh():
+            full_dest = str(self.destination.login) + ':' + str(self.current_backup_path)
+            command.append(full_dest)
+        else:
+            command.append(self.current_backup_path)
 
         if self.filter:
             # Add filters, the resulting command must look like
@@ -304,33 +377,15 @@ class Job:
         self.logger.debug("rsync command: %s", command)
         return command
 
-#    def _compress_increments(self):
-#        """
-#        Compress increments and delete old ones
-#        """
-#        if self.dest_type == 'DIR':
-#            #compress if not empty
-#            if os.listdir(self.inc_path) != []:
-#                self.logger.debug("Zip the directory: %s", self.inc_path)
-#                utils.compress(self.inc_path)
-#            else:
-#                self.logger.info('Empty increment')
-#                pass
-#            #delete the dir (we keep only non-empty tarballs
-#            shutil.rmtree(self.inc_path)
-#
-#            #MrProper: remove old tarballs
-#            self._delete_old_files(self.inc_dir, days=self.duration, keep=self.keep)
-#        elif self.dest_type == 'SSH':
-#            pass
-#            #TODO ! compress dir though ssh
-#            #TODO Remove old dirs though ssh
-
-    def _rsync(self):
+    def _run_command(self, command):
         """ 
-        Run rsync to do the task.
+        Run a command and log stderr+stdout in a dedicated log file.
+        :param command: a list, each element is a part of the command line
+
+        Example
+        -------
+        command = ['/usr/bin/cp', '-r', '/home', '/tmp']
         """
-        command = self._prepare_rsync_command()
 
         #If a signal is received, stop the process
         #if self.terminate: return
@@ -351,32 +406,37 @@ class Job:
         """
         Run the job
         """
+        #TODO rewriting and integration
+        #self._check_disk_usage()
         if self._check_need_backup():
+            self.job_logger.info('='*20 + str(self.now) + '='*20)
             print(self.name)
             self.logger.info("Backup %s", self.name)
             #Prepare the destination
             self._prepare_destination()
-            self.logger.debug("source path %s", self.source) 
-            self.logger.debug("destination path %s", self.destination) 
+            self.logger.debug("source path %s", self.source.target) 
+            self.logger.debug("destination path %s", self.destination.target) 
             self.logger.debug("filter path %s", self.filter)
             #Run rsync
-            self._rsync()
-            #Compress
+            command = self._prepare_rsync_command()
+            self._run_command(command)
             #Job done, update the time in the database
             self._set_lastbackup_time()
-        #Remove old snapshots, local only
-        if self.dest_type == 'DIR' and self.snapshot:
-            dest = os.path.join(self.destination, self.name)
-            self._delete_old_files(dest, days=self.duration, keep=self.keep)
-            #Symlink
-            last = os.path.join(self.destination , self.name, 'last')
-            if os.path.islink(last):
-                os.remove(last)
-            try:
-                os.symlink(self.current_backup_path, last)
-            except FileExistsError:
-                self.logger.warn('The symlink %s could not be created because a file exists', last)
-            except AttributeError:
-                self.logger.warn('Attribute error for symlink') #FIXME
+            #Remove old snapshots
+            #self._delete_old_files(days=self.duration, keep=self.keep)
+            self._delete_old_files(days=0, keep=3)
+            #Create symlink TODO: SSH
+            if self.destination.is_dir() and self.snapshot:
+                #Symlink
+                last = os.path.join(self.destination.path, self.name, 'last')
+                if os.path.islink(last):
+                    os.remove(last)
+                os.chdir(os.path.dirname(self.current_backup_path))
+                try:
+                    os.symlink(os.path.basename(self.current_backup_path), last)
+                except FileExistsError:
+                    self.logger.warn('The symlink %s could not be created because a file exists', last)
+                except AttributeError:
+                    self.logger.warn('Attribute error for symlink. Job: %s', self.name) 
 
 
